package com.ruoyi.sim.service.impl;

import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.sim.config.SimConfig;
import com.ruoyi.sim.constant.CommConst;
import com.ruoyi.sim.domain.Seat;
import com.ruoyi.sim.domain.vo.SimSocketVo;
import com.ruoyi.sim.domain.vo.SocketWrapValue;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.net.Socket;
import java.util.HashMap;
import java.util.List;

import static com.ruoyi.sim.constant.CommConst.SOCKET_TIME_OUT;

/**
 * 1.所有Socket常开。
 * todo: connectedTimeMillis 时间。
 */
@Service
public class SocketService {

    private static final Logger l = LoggerFactory.getLogger(SocketService.class);

    private static final int INIT_SIZE = 32;
    /**
     * 6 hours
     * 1000L * 60 * 60 * 6
     * 1000L * 60 * 5
     */
    private static final long TIMEOUT_LIMIT = 1000L * 60 * 60 * 6;

    /**
     * key: ip:port
     * value:
     */
    private static HashMap<String, SocketWrapValue> cachedMap = new HashMap<>(INIT_SIZE);

    @Autowired
    private SimConfig config;
    @Autowired
    private FailedCountService failedCountService;
    @Autowired
    private SeatService seatService;

    /**
     * @param ip   ip v4.
     * @param port
     * @return true:socket ok!
     */
    public boolean isOk(final String ip, final Integer port) {
        final String key = buildKey(ip, port);
        if (cachedMap.containsKey(key)) {
            Socket s = cachedMap.get(key).getSocket();
            if (s != null) {
                return (s.isConnected() && s.isBound() && !s.isClosed() && !isTimeout(ip, port));
            }
        }
        return false;
    }

    public boolean isNotOk(final String ip, final Integer port) {
        return !isOk(ip, port);
    }

    public boolean isTimeout(final String ip, final Integer port) {
        final String key = buildKey(ip, port);
        if (cachedMap.containsKey(key)) {
            Long cached = cachedMap.get(key).getOkTimeMillis();
            if (cached == null || cached == 0L) {
                return true;
            }
            if (System.currentTimeMillis() - cached > TIMEOUT_LIMIT) {
                return false;
            }
        }
        return true;
    }

    /**
     * @param ip   ip v4.
     * @param port
     * @return
     */
    public String buildKey(final String ip, final Integer port) {
        if (StringUtils.isBlank(ip)) {
            throw new IllegalArgumentException("ip error");
        }
        if (port == null || port <= CommConst.PORT_MIN || port >= CommConst.PORT_MAX) {
            throw new IllegalArgumentException("port error");
        }
        return ip + ":" + port;
    }

    /**
     * @param ssv
     * @return
     */
    public AjaxResult openOne(final SimSocketVo ssv) {
        // check.
        if (!config.isCommGlobal()) {
            l.warn("isCommGlobal == {} [模拟器通信被禁用！]", config.isCommGlobal());
            return AjaxResult.error("模拟器通信被禁用！");
        }
        //
        try {
            final String ip = ssv.getIp();
            final Integer port = ssv.getPort();
            final String key = buildKey(ip, port);
            if (isNotOk(ip, port)) {
                l.info("openSocket cachedSocket is not ok!new socket ip = {}:{}!", ip, port);
                closeOne(ssv);
                Socket s = new Socket(ip, port);
                s.setSoTimeout(SOCKET_TIME_OUT);
                SocketWrapValue value = new SocketWrapValue(ip, port, s, System.currentTimeMillis());
                cachedMap.put(key, value);
                // failed count reset.
                failedCountService.reset0(key);
            } else {
                l.info("openSocket cachedSocket cache ok!cached socket ip = {}:{}!", ip, port);
            }
        } catch (IOException e) {
            l.error(ssv.toString());
            throw new RuntimeException(e);
        } finally {

        }
        return AjaxResult.success("openOneSocket Success！");
    }

    /**
     * todo:部分返回Aj结果。
     *
     * @return
     */
    public AjaxResult openAll() {
        if (!config.isCommGlobal()) {
            l.warn("isCommGlobal == {} [模拟器通信被禁用！]", config.isCommGlobal());
            return AjaxResult.error("模拟器通信被禁用！");
        }
        List<Seat> allSeat = seatService.listAllEnable();
        for (Seat s : allSeat) {
            openOne(new SimSocketVo(s.getSeatRs485Ip(), s.getSeatRs485Port()));
        }
        return AjaxResult.success("openAllSocket Success！");
    }

    public AjaxResult closeOne(final SimSocketVo ssv) {
        if (!config.isCommGlobal()) {
            l.warn("isCommGlobal == {} [模拟器通信被禁用！]", config.isCommGlobal());
            return AjaxResult.error("模拟器通信被禁用！");
        }
        String msgOk = "关闭Socket成功！";
        final String key = buildKey(ssv.getIp(), ssv.getPort());
        try {
            if (cachedMap.containsKey(key)) {
                Socket s = cachedMap.get(key).getSocket();
                s.getInputStream().close();
                s.getOutputStream().close();
                s.close();
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            cachedMap.remove(key);
            // failed count reset.
            failedCountService.reset0(key);
            return AjaxResult.success(msgOk);
        }
    }

    public AjaxResult closeAll() {
        if (!config.isCommGlobal()) {
            l.warn("isCommGlobal == {} [模拟器通信被禁用！]", config.isCommGlobal());
            return AjaxResult.error("模拟器通信被禁用！");
        }
        final String msgOk = "关闭所有Socket成功！";
        List<Seat> allSeat = seatService.listAllEnable();
        for (Seat s : allSeat) {
            closeOne(new SimSocketVo(s.getSeatRs485Ip(), s.getSeatRs485Port()));
        }
        return AjaxResult.success(msgOk);
    }

    /**
     * @param ip
     * @param port
     * @return
     */
    public Socket get(final String ip, final Integer port) {
        final String key = buildKey(ip, port);
        if (isNotOk(ip, port)) {
            openOne(new SimSocketVo(ip, port));
        }
        return cachedMap.get(key).getSocket();
    }

    /**
     * 初始化。
     */
    public void resetAll() {

    }
}
